@@ -58,7 +58,13 @@ typedef enum
   META_MENU_OP_STICK       = 1 << 7,
   META_MENU_OP_WORKSPACES  = 1 << 8,
   META_MENU_OP_MOVE        = 1 << 9,
-  META_MENU_OP_RESIZE      = 1 << 10
+  META_MENU_OP_RESIZE      = 1 << 10,
+  META_MENU_OP_ABOVE       = 1 << 11,
+  META_MENU_OP_UNABOVE     = 1 << 12,
+  META_MENU_OP_MOVE_LEFT   = 1 << 13,
+  META_MENU_OP_MOVE_RIGHT  = 1 << 14,
+  META_MENU_OP_MOVE_UP     = 1 << 15,
+  META_MENU_OP_MOVE_DOWN   = 1 << 16
 } MetaMenuOp;
 
 typedef struct _MetaWindowMenu MetaWindowMenu;
