@@ -34,6 +34,7 @@
 #include "keybindings.h"
 #include "stack.h"
 #include "xprops.h"
+#include "compositor.h"
 
 #ifdef HAVE_SOLARIS_XINERAMA
 #include <X11/extensions/xinerama.h>
@@ -81,7 +82,7 @@ set_wm_check_hint (MetaScreen *screen)
 static int
 set_supported_hint (MetaScreen *screen)
 {
-#define N_SUPPORTED 49
+#define N_SUPPORTED 54
   Atom atoms[N_SUPPORTED];
   
   atoms[0] = screen->display->atom_net_wm_name;
@@ -133,6 +134,11 @@ set_supported_hint (MetaScreen *screen)
   atoms[46] = screen->display->atom_net_wm_state_above;
   atoms[47] = screen->display->atom_net_wm_state_below;
   atoms[48] = screen->display->atom_net_startup_id;
+  atoms[49] = screen->display->atom_net_wm_strut_partial;
+  atoms[50] = screen->display->atom_net_wm_action_fullscreen;
+  atoms[51] = screen->display->atom_net_wm_action_minimize;
+  atoms[52] = screen->display->atom_net_frame_extents;
+  atoms[53] = screen->display->atom_net_request_frame_extents;
   
   XChangeProperty (screen->display->xdisplay, screen->xroot,
                    screen->display->atom_net_supported,
@@ -356,7 +362,8 @@ reload_xinerama_infos (MetaScreen *screen)
 
 MetaScreen*
 meta_screen_new (MetaDisplay *display,
-                 int          number)
+                 int          number,
+                 Time         timestamp)
 {
   MetaScreen *screen;
   Window xroot;
@@ -422,24 +429,7 @@ meta_screen_new (MetaDisplay *display,
 
   new_wm_sn_owner = meta_create_offscreen_window (xdisplay, xroot);
 
-  {
-    /* Generate a timestamp */
-    XSetWindowAttributes attrs;
-    XEvent event;
-
-    attrs.event_mask = PropertyChangeMask;
-    XChangeWindowAttributes (xdisplay, new_wm_sn_owner, CWEventMask, &attrs);
-    
-    XChangeProperty (xdisplay,
-                     new_wm_sn_owner, XA_WM_CLASS, XA_STRING, 8,
-                     PropModeAppend, NULL, 0);
-    XWindowEvent (xdisplay, new_wm_sn_owner, PropertyChangeMask, &event);
-    attrs.event_mask = NoEventMask;
-    XChangeWindowAttributes (display->xdisplay,
-                             new_wm_sn_owner, CWEventMask, &attrs);
-
-    manager_timestamp = event.xproperty.time;
-  }
+  manager_timestamp = timestamp;
   
   XSetSelectionOwner (xdisplay, wm_sn_atom, new_wm_sn_owner,
                       manager_timestamp);
@@ -498,6 +488,9 @@ meta_screen_new (MetaDisplay *display,
                 LeaveWindowMask | EnterWindowMask |
                 KeyPressMask | KeyReleaseMask |
                 FocusChangeMask | StructureNotifyMask |
+#ifdef HAVE_COMPOSITE_EXTENSIONS
+                ExposureMask |
+#endif
 		attr.your_event_mask);
   if (meta_error_trap_pop_with_return (display, FALSE) != Success)
     {
@@ -538,19 +531,34 @@ meta_screen_new (MetaDisplay *display,
   screen->starting_corner = META_SCREEN_TOPLEFT;
 
   screen->showing_desktop = FALSE;
+
+  screen->compositor_windows = NULL;
+  screen->damage_region = None;
+  screen->root_picture = None;
+  screen->trans_pixmap = None;
+  screen->trans_picture = None;
+  
+  {
+    XGCValues gc_values;
+    
+    gc_values.subwindow_mode = IncludeInferiors;
+    gc_values.function = GXinvert;
+    gc_values.line_width = META_WIREFRAME_XOR_LINE_WIDTH;
+    
+    screen->root_xor_gc = XCreateGC (screen->display->xdisplay,
+                                     screen->xroot,
+                                     GCSubwindowMode | GCFunction | GCLineWidth,
+                                     &gc_values);
+  }
   
   screen->xinerama_infos = NULL;
   screen->n_xinerama_infos = 0;
-  screen->last_xinerama_index = 0;
+  screen->last_xinerama_index = 0;  
   
   reload_xinerama_infos (screen);
   
   meta_screen_set_cursor (screen, META_CURSOR_DEFAULT);
   
-  if (display->leader_window == None)
-    display->leader_window = meta_create_offscreen_window (display->xdisplay,
-                                                           screen->xroot);
-  
   if (display->no_focus_window == None)
     {
       display->no_focus_window = meta_create_offscreen_window (display->xdisplay,
@@ -622,9 +630,12 @@ meta_screen_new (MetaDisplay *display,
     if (space != NULL)
       meta_workspace_activate (space);
   }
+
+  meta_compositor_manage_screen (screen->display->compositor,
+                                 screen);
   
   meta_verbose ("Added screen %d ('%s') root 0x%lx\n",
-                screen->number, screen->screen_name, screen->xroot);  
+                screen->number, screen->screen_name, screen->xroot);
   
   return screen;
 }
@@ -640,6 +651,9 @@ meta_screen_free (MetaScreen *screen)
   
   meta_display_grab (display);
 
+  meta_compositor_unmanage_screen (screen->display->compositor,
+                                   screen);
+  
   meta_display_unmanage_windows_for_screen (display, screen);
   
   meta_prefs_remove_listener (prefs_changed_callback, screen);
@@ -679,6 +693,9 @@ meta_screen_free (MetaScreen *screen)
   
   if (screen->work_area_idle != 0)
     g_source_remove (screen->work_area_idle);
+
+  XFreeGC (screen->display->xdisplay,
+           screen->root_xor_gc);
   
   g_free (screen->screen_name);
   g_free (screen);
@@ -714,7 +731,26 @@ meta_screen_manage_all_windows (MetaScreen *screen)
   i = 0;
   while (i < n_children)
     {
-      meta_window_new (screen->display, children[i], TRUE);
+      XWindowAttributes attrs;
+      
+      meta_error_trap_push_with_return (screen->display);
+      
+      XGetWindowAttributes (screen->display->xdisplay,
+                            children[i], &attrs);
+      
+      if (meta_error_trap_pop_with_return (screen->display, TRUE) != Success)
+        {
+          meta_verbose ("Failed to get attributes for window 0x%lx\n",
+                        children[i]);
+        }
+      else
+        {
+          meta_window_new_with_attrs (screen->display, children[i], TRUE,
+                                      &attrs);
+
+          meta_compositor_add_window (screen->display->compositor,
+                                      children[i], &attrs);
+        }
 
       ++i;
     }
@@ -1572,6 +1608,8 @@ meta_screen_update_workspace_layout (MetaScreen *screen)
                     break;
                 }
             }
+          else
+            screen->starting_corner = META_SCREEN_TOPLEFT;
         }
       else
         {
@@ -1679,6 +1717,7 @@ meta_create_offscreen_window (Display *xdisplay,
    * (but on a display we are managing at least one screen for)
    */
   attrs.override_redirect = True;
+  attrs.event_mask = PropertyChangeMask;
   
   return XCreateWindow (xdisplay,
                         parent,
@@ -1687,7 +1726,7 @@ meta_create_offscreen_window (Display *xdisplay,
                         CopyFromParent,
                         CopyFromParent,
                         CopyFromParent,
-                        CWOverrideRedirect,
+                        CWOverrideRedirect | CWEventMask,
                         &attrs);
 }
 
