@@ -2,6 +2,7 @@
 
 /* 
  * Copyright (C) 2001 Havoc Pennington
+ * Copyright (C) 2003 Rob Adams
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -239,7 +240,7 @@ gboolean
 meta_workspace_contains_window (MetaWorkspace *workspace,
                                 MetaWindow    *window)
 {
-  return g_list_find (workspace->windows, window) != NULL;
+  return g_list_find (window->workspaces, workspace) != NULL;
 }
 
 void
